Attribute VB_Name = "MathBetaF"
Option Explicit
'Example Usage of BetI Function
'
'Student's distribution A(t|Nu)
'
'   A(t|Nu) = 1! - BetaI(Nu / 2!, 0.5!, Nu / (Nu + t * t))
'
'F-Distribution Function Q(F|Nu1,Nu2)
'
'   Q(F|Nu1,Nu2) = BetaI(Nu2 / 2!, Nu1 / 2!, Nu2 / (Nu2 + Nu1 * F))

Public Function BetaI(ByVal A As Single, _
                      ByVal B As Single, _
                      ByVal x As Single) As Single
    Dim Bt As Single
    
    If x < 0! Or x > 1! Then Err.Raise 5
    If A <= 0! Or B <= 0! Then Err.Raise 5
    
    If x = 0! Or x = 1! Then
        Bt = 0!
    Else
        Bt = Exp(GammLn(A + B) - GammLn(A) - GammLn(B) _
                    + A * Log(x) + B * Log(1! - x))
    End If
    
    If x < (A + 1!) / (A + B + 2!) Then
        BetaI = Bt * BetaCf(A, B, x) / A
    Else
        BetaI = 1! - Bt * BetaCf(B, A, 1! - x) / B
    End If
            
End Function

Private Function BetaCf(ByVal A As Single, _
                        ByVal B As Single, _
                        ByVal x As Single) As Single
    Const IT_MAX As Integer = 100
    Const EPS As Single = 0.0000003!
    
    Dim QaP As Single, QaM As Single, Qab As Single
    Dim Em As Single, TEm As Single, D As Single
    
    Dim Bz As Single, Bm As Single
    Dim BP As Single, BPP As Single
    
    Dim Az As Single, Am As Single
    Dim Ap As Single, APP As Single, Aold As Single
    
    Dim M As Long
    
    
    Bm = 1!: Az = 1!: Am = 1!
    
    Qab = A + B
    QaP = A + 1!
    QaM = A - 1!
    Bz = 1! - Qab * x / QaP
    
    For M = 1 To IT_MAX
        Em = CSng(M)
        TEm = Em + Em
        
        D = Em _
                * (B - Em) _
                * x / ((QaM + TEm) * (A + TEm))
        
        Ap = Az + D * Am
        BP = Bz + D * Bm
        
        D = -(A + Em) _
                * (Qab + Em) _
                * x / ((QaP + TEm) * (A + TEm))
        
        APP = Ap + D * Az
        BPP = BP + D * Bz
        
        Aold = Az
        Am = Ap / BPP
        Bm = BP / BPP
        Az = APP / BPP
        Bz = 1!
        
        If Abs(Az - Aold) < EPS * Abs(Az) Then
            BetaCf = Az
            Exit Function
        End If
    Next M
    
    Err.Raise 5
End Function
